#!/bin/sh
#
# Component: checkintegrity
#
# Description:
#   Collects list of studies to be checked. Lists any study found in data, processed, cache directories and SQL tables.
#
# Usage:
#    collectStudyList.sh [-v]
#
#   -v     : print debug messages
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

isDebug="false"

if [ "$1" = "-v" ] ; then
	isDebug="true"
	shift
fi

# Undocumented feature: pid will be used for creating the working directory.
# pid can also be specified from command line: this is useful for developing/testing the script
pid="$1"

[ -z "$pid" ] && pid=$$

wdir="$TMP/chstudies.$pid"


# Prints debug message
#
# $1 message to be printed
#
debug_()
{
	[ "$isDebug" != "true" ] && return
	[ -n "$1" ] && echo "DEBUG: $1"
}

#
# Lists studies in a repository
#
# $1 - filename to be created
# $2 - repository to be listed
#
listRepository_()
{
	if [ -z "$1" ] ; then
		echo "Missing argument (1) for listRepository_()" >&2
		return 1
	fi
	if [ -z "$2" ] ; then
		echo "Missing argument (2) for listRepository_()" >&2
		return 1
	fi

	debug_ "Collecting studies from $2 to $1"
	if [ -f "$1" ] ; then
		debug_ "$1 already exists; skipped"
		return 0;
	fi

	$REPOSITORYHANDLER_HOME/bin/listDirectories "$2" | sort > $1
}


#
# Collects the list of studies in the data repository.
# This will be the main study list
#
collectDicomLS_()
{
	if [ -f "$wdir/dicom.ls.sorted" ] ; then
		debug_ "$wdir/dicom.ls.sorted already exists"
		return 0;
	fi

	listRepository_ "$wdir/dicom.ls" "$REPOSITORY_DICOM_DATA"

	sort $wdir/dicom.ls > $wdir/dicom.ls.sorted
}


# List directory
#
# $1 directory
# $2 level
listDir_()
{
#	debug_ "List Directory $1  [$2]"
	if [ $2 -eq 0 ] ; then
		ls "$1/"
		return 0
	fi

	for d in `ls $1/` ; do
		listDir_ "$1/$d" $(( $2 - 1 ))
	done
}


# Collect studies from a directory structure
#
# $1 - filename to be created
# $2 - directory structure to be listed
# [$3] - depth of the directroy structure
# 
listSorted_()
{
	if [ -z "$1" ] ; then
		echo "Missing argument (1) for listSorted_()" >&2
		return 1
	fi
	if [ -z "$2" ] ; then
		echo "Missing argument (2) for listSorted_()" >&2
		return 1
	fi
	if [ ! -d "$2" ] ; then
		# legacy directories will not be created on brand new servers
		debug_ "Directory does not exist"
		return 0
	fi

	debug_ "Collecting studies from $2 to $1"
	if [ -f "$1" ] ; then
		debug_ "$1 already exists; skipped"
		return 0;
	fi

	depth=$3
	if [ -z "$depth" ] ; then
		listDir_ "$2" 0
	else
		listDir_ "$2" 0 | grep -v "\<..\>"
		listDir_ "$2" $depth
	fi | sort > $1
}

# Collect studies from database table
#
# $1 - filename to be created
# $2 - table name
#
listDatabase_()
{
	if [ -z "$1" ] ; then
		echo "Missing argument (1) for listDatabase_()" >&2
		return 1
	fi
	if [ -z "$2" ] ; then
		echo "Missing argument (2) for listDatabase_()" >&2
		return 1
	fi

	debug_ "Collecting studies from database table $2 to $1"
	if [ -f "$1" ] ; then
		debug_ "$1 already exists; skipped"
		return 0;
	fi

	local fldName="STYIUID"
	[ $2 == "Dcreport" ] && fldName="SRCSTUDY"

	echo "select distinct $fldName from $2;" | $MYSQL_HOME/bin/mysql -N $DBNAME | sort > $1
}


# Merge list files created by listSorted_ and listDatabase_
#
mergeListFiles_()
{
	debug_ "Merge lists of studies to $wdir/merge.ls"
	if [ -f "$wdir/merge.ls" ] ; then
		debug_ "$wdir/merge.ls already exists; skipped"
		return 0;
	fi

	sort -u -m $wdir/*.tbm.ls > $wdir/merge.ls
}


# Prints the list of studies to be checked to the standard output
# Studies missing from dicom.ls (data/dicom) will be printed first
# followed by the studies in dicom.ls from the oldest to the newest.
#
printStudyList_()
{
	debug_ "diff $wdir/merge.ls $wdir/dicom.ls.sorted | awk '($1 == \">\") {print $2}'"
	diff $wdir/dicom.ls.sorted $wdir/merge.ls | awk '($1 == ">") {print $2}'
	debug_ "cat $wdir/dicom.ls"
	cat $wdir/dicom.ls
}

debug_ "Working Directory is $wdir"

if [ ! -d "$wdir" ] ; then
	mkdir $wdir
	if [ $? -ne 0 ] ; then
		echo "$wdir could not be created" >&2
		exit 1
	fi
fi

######
# DATA
######

# creates dicom.ls and dicom.ls.sorted
collectDicomLS_

# legacy data directories
listSorted_ "$wdir/dicom.legacy.tbm.ls" "$DICOM_DATA"
listSorted_ "$wdir/dicom.hashed.tbm.ls" "$HASHED_DICOM_DATA" 3

###########
# PROCESSED
###########
listRepository_ "$wdir/processed.tbm.ls" "$REPOSITORY_PROCESSED"

# Legacy Processed directories
listSorted_ "$wdir/processed.legacy.tbm.ls" "$PROCESSED"
listSorted_ "$wdir/processed.hashed.tbm.ls" "$HASHED_PROCESSED" 3

#######
# CACHE
#######
listRepository_ "$wdir/cache.tbm.ls" "$REPOSITORY_CACHE"

listSorted_ "$wdir/cache.legacy.tbm.ls" "$CACHE"
listSorted_ "$wdir/cache.hashed.tbm.ls" "$HASHED_CACHE" 3

####################
# Reference Counters
####################
listSorted_ "$wdir/refcount.studydir.tbm.ls" "$REFCOUNT_VAR/STUDYDIR"
listSorted_ "$wdir/refcount.studydb.tbm.ls" "$REFCOUNT_VAR/STUDYDB"
listSorted_ "$wdir/refcount.objectfile.tbm.ls" "$REFCOUNT_VAR/OBJECTFILE" 3

########################
# Database table entries
########################
listDatabase_ "$wdir/dcstudy.tbm.ls" $STUDYTBL
listDatabase_ "$wdir/dcobject.tbm.ls" $OBJECTTBL
listDatabase_ "$wdir/dcreport.tbm.ls" $REPORTTBL

# creates merge.ls
mergeListFiles_

printStudyList_

exit 0
