#!/bin/sh
#
# Component: dotcom
#
# Description:
#   Proliferates a file in the dotcom.
#
# Usage:
#    proliferate.sh -f <filename> [-n] [-d <direction>] [-x <excepthost>]
#         [[-r <script>] [-a <scriptarg>]]
#         [-R <refcount type> [-I <refcount ID>]]
#
#      -f <filename>      : file to proliferate
#      -n                 : do not proliferate to Database servers
#      -x <excepthost>    : don't send the file to this host
#      -d <direction>     : direction of proliferate {up|down|all|backup|all-backup}, default is all-backup
#      -r <script>        : run this script on the target
#      -a <scriptarg>     : params passed to script
#      -R <refcount type> : Add/Remove <refcount type> references before/after uploads
#      -I <refcount ID>   : Refcount identifier (= `basename <filename>` in default)
#
# If -R option is provided it's supposed that a 'proliferate' reference has been added to
# the refcount ID (file) before. This reference will be removed at the end of this script
#
#
# Last Updated     : $Author: pzahorcsek $
# Update Date Time : $Date: 2014/03/13 15:28:57 $
# RCS File Name    : $RCSfile: proliferate.sh,v $
# RCS Revision     : $Revision: 1.19 $
# Locked by        : $Locker:  $
# Symbolic name    : $Name:  $
# State            : $State: Exp $
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="proliferate[$$]"

#
# Usage
#
usage_()
{
	if [ -n "$1" ] ; then
		echo $1
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$1"
	fi
	cat <<- ENDOFUSAGE
		proliferate.sh -f <filename> [-n] [-d <direction>] [-x <excepthost>]
		     [[-r <script>] [-a <scriptarg>]]
		     [-R <refcount type> [-I <refcount ID>]]
	
		 -f <filename>      : file to proliferate
		 -n                 : do not proliferate to Database servers
		 -x <excepthost>    : don't send the file to this host
		 -d <direction>     : direction of proliferate {up|down|all|backup|all-backup}, default is all-backup
		 -r <script>        : run this script on the target
		 -a <scriptarg>     : params passed to script
		 -R <refcount type> : Add/Remove <refcount type> references before/after uploads
		 -I <refcount ID>   : Refcount identifier (= \`basename <filename>\` in default)
	
		If -R option is provided it's supposed that a 'proliferate' reference has been added to
		the refcount ID (file) before. This reference will be removed at the end of this script
	
	ENDOFUSAGE
} # usage


# proliferate.sh can be called directly:
# run as a task instead
if [ `basename $0` = "proliferate.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
#	$TASKD_HOME/runshellcmd.sh -R $0 "$@"
	exit $?
fi


ERRMSG=""
FILENAME=""
DIRECTION=""
DEFAULTDIRECTION="all-backup"
EXCEPT=""

PEXCEPT=""
PSCRIPTNAME=""
PSCRIPTARGS=""

REFCOUNTTYPE=""
REFCOUNTID=""

EXCLUDE_DBONLY=""

while getopts 'f:x:d:r:a:R:I:n' OPT; do
	case $OPT in
		f) FILENAME="$OPTARG" ;;
		x) EXCEPT="$OPTARG" ;;
		d) DIRECTION="$OPTARG" ;;
		r) PSCRIPTNAME="-r \"$OPTARG\"" ;;
		a) PSCRIPTARGS="$PSCRIPTARGS -a \"$OPTARG\"" ;;
		R) REFCOUNTTYPE="$OPTARG";;
		I) REFCOUNTID="$OPTARG" ;;
		n) EXCLUDE_DBONLY="-n" ;;
		*) ERRMSG="Invalid_Parameter:$OPT" ;;
	esac
done

[ -z "$FILENAME" ] && ERRMSG="Missing Parameter"

run_() # override run_ method
{
	if [ -n "$ERRMSG" ] ; then
		usage_ "$ERRMSG"
		return 125
	fi

	WHERECLAUSE=""
	if [ "$DIRECTION" != "all" -a "$DIRECTION" != "up" -a "$DIRECTION" != "down" -a "$DIRECTION" != "backup" -a "$DIRECTION" != "all-backup" ] ; then
		DIRECTION="$DEFAULTDIRECTION"
	fi
	
	if [ "$DIRECTION" = "all" ] ; then
		WHERECLAUSE="where (TYPE='P' OR TYPE='C')"
	elif [ "$DIRECTION" = "backup" -o "$DIRECTION" = "all-backup" ] ; then
		all=""
		[ "$DIRECTION" = "all-backup" ] && all=" OR TYPE='P' OR TYPE='C'"
		WHERECLAUSE="where (TYPE='B'$all)"
	elif [ "$DIRECTION" = "down" ] ; then
		WHERECLAUSE="where TYPE='C'"
	elif [ "$DIRECTION" = "up" ] ; then
		WHERECLAUSE="where TYPE='P'"
	fi

	if [ -n "$EXCEPT" ] ; then
		WHERECLAUSE="$WHERECLAUSE AND ID!='$EXCEPT' AND HOST!='$EXCEPT'"
	fi

	if [ -n "$EXCLUDE_DBONLY" ] ; then
		dbOnly=`$DOTCOM_HOME/getDatabaseDevice.awk $CONF/dotcom.jcfg`
		if [ -n "$dbOnly" ] ; then
			for dbDev in $dbOnly ; do
				WHERECLAUSE="$WHERECLAUSE AND HOST!='$dbDev'"
			done
		fi
	fi
	
	HostIDs=`echo "select ID from $TARGETTBL $WHERECLAUSE;" | $MYSQL_HOME/bin/mysql -N $DBNAME`

	if [ $? -ne 0 ] ; then
	# retry
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "Proliferate $FILENAME MySQL error"
		return 1
	fi

	DATE=`date +%Y%m%d%H%M%S`
	PID=$$

	[ -z "$PSCRIPTNAME" ] && PSCRIPTNAME="-r $DOTCOM_HOME/applystore.sh"
	PSCRIPTARGS="$PSCRIPTARGS -a -d -a \"$DIRECTION\""
	[ -n "$EXCLUDE_DBONLY" ] && PSCRIPTARGS="$PSCRIPTARGS -a -n"

	REFCOUNTPARAMS=""
	if [ -n "$REFCOUNTTYPE" ] ; then
		REFCOUNTPARAMS="-R \"$REFCOUNTTYPE\""
		if [ -n "$REFCOUNTID" ] ; then
#			REFCOUNTID=`basename ${FILENAME}`
#		else
			REFCOUNTPARAMS="$REFCOUNTPARAMS -I \"$REFCOUNTID\""
		fi
	fi

	for host in $HostIDs ; do
		# start sending task

		if [ -n "$REFCOUNTTYPE" ] ; then
			[ -z "$REFCOUNTID" ] && REFCOUNTID=`basename ${FILENAME}`
			$UTILS_HOME/bin/refcount -a "uploadto_$host" -i "${REFCOUNTID}" -t "${REFCOUNTTYPE}" >>$LOG/refcount.error 2>&1
		fi

		$TASKD_HOME/createchildtask.sh -p "UPLOAD" -t "class" -c "dotcom.Upload -f \"$FILENAME\" -H $host $REFCOUNTPARAMS $PSCRIPTNAME $PSCRIPTARGS"
	done

	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Proliferate $FILENAME started"

	return 0
}

print_()
{
	echo "Proliferates $FILENAME in dotcom"
}

cleanup_()
{
	if [ -n "$REFCOUNTTYPE" ] ; then
		[ -z "$REFCOUNTID" ] && REFCOUNTID=`basename ${FILENAME}`
		$UTILS_HOME/bin/refcount -r "proliferate" -i "${REFCOUNTID}" -t "${REFCOUNTTYPE}" >>$LOG/refcount.error 2>&1
	fi
}


#
# default get display name (for subjob)
#
get_display_name_()
{
	echo "Proliferates a config or data file in the hierarchy"
}


get_resources_()
{
	echo "System.NET"
}


get_watson_message_()
{
	ID_FILENAME=`echo "$FILENAME" | sed 's/[^a-zA-Z0-9\.]/_/g'`
	[ -z "$DR_WATSON_ID" ] && DR_WATSON_ID="proliferate.$ID_FILENAME"
	[ -z "$DR_WATSON_MESSAGE" ] && DR_WATSON_MESSAGE="Starting proliferate $FILENAME"

	if [ "$RUN_RESULT" -gt 124 ] ; then
		WATSON_MESSAGE_ID="$DR_WATSON_ID.failed"
		WATSON_MESSAGE_TEXT="FAILED: $DR_WATSON_MESSAGE"
		return
	fi

	if [ "$RUN_RESULT" -ne 0 -a "$RETRY_NUM" -gt 0 -a $(( $RETRY_NUM % 10)) -eq 0 ] ; then
		WATSON_MESSAGE_TEXT="ERROR: $DR_WATSON_MESSAGE [Retry #$RETRY_NUM]"
		WATSON_MESSAGE_ID="$DR_WATSON_ID.$RETRY_NUM"
	fi

} # get_watson_message_()

